'use strict';

var _defineProperty = require('@babel/runtime/helpers/defineProperty');
var variables = require('../variables.js');

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

/**
 * @param {MinterApiInstance} apiInstance
 * @param {import('axios').AxiosRequestConfig} [factoryAxiosOptions]
 * @return {GetNonceInstance}
 */
function GetNonce(apiInstance, factoryAxiosOptions) {
  /**
   * Get nonce for new transaction: last transaction number + 1
   * @typedef {Function} GetNonceInstance
   * @param {string} address
   * @param {import('axios').AxiosRequestConfig} [axiosOptions]
   * @return {Promise<number>}
   */
  return function getNonce(address, axiosOptions) {
    var nonceUrl = apiInstance.defaults.apiType === variables.API_TYPE_GATE ? "nonce/".concat(address) : "address/".concat(address);
    return apiInstance.get(nonceUrl, _objectSpread(_objectSpread({}, factoryAxiosOptions), axiosOptions)).then(function (response) {
      var resData = response.data;
      var nonce = apiInstance.defaults.apiType === variables.API_TYPE_GATE ? resData.nonce : resData.transaction_count;
      var newNonce = Number(nonce) + 1;
      if (Number.isNaN(newNonce)) {
        // eslint-disable-next-line unicorn/prefer-type-error
        throw new Error("Can't get nonce, API returns '".concat(nonce, "'"));
      }
      return newNonce;
    });
  };
}

module.exports = GetNonce;
