'use strict';

var _typeof = require('@babel/runtime/helpers/typeof');
var _baseGetTag = require('./_baseGetTag.js');
var isObjectLike = require('./isObjectLike.js');

/** `Object#toString` result references. */
var symbolTag = '[object Symbol]';

/**
 * Checks if `value` is classified as a `Symbol` primitive or object.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
 * @example
 *
 * _.isSymbol(Symbol.iterator);
 * // => true
 *
 * _.isSymbol('abc');
 * // => false
 */
function isSymbol(value) {
  return _typeof(value) == 'symbol' || isObjectLike(value) && _baseGetTag(value) == symbolTag;
}

module.exports = isSymbol;
