'use strict';

var minterjsTx = require('minterjs-tx');
var minterjsUtil = require('minterjs-util');
var utils = require('../utils.js');

/**
 * @param {string} publicKey
 * @param {TxOptions} [options]
 * @constructor
 */
function SetCandidateOnTxData(_ref) {
  var publicKey = _ref.publicKey;
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  if (!options.disableValidation) {
    utils.validatePublicKey(publicKey, 'publicKey');
  }
  this.publicKey = publicKey;
  this.txData = new minterjsTx.TxDataSetCandidateOn({
    publicKey: minterjsUtil.toBuffer(publicKey)
  });
  utils.proxyNestedTxData(this);
}

/**
 * @param {Buffer|string} publicKey
 * @param {TxOptions} [options]
 * @return {SetCandidateOnTxData}
 */
SetCandidateOnTxData.fromBufferFields = function fromBufferFields(_ref2) {
  var publicKey = _ref2.publicKey;
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return new SetCandidateOnTxData({
    publicKey: utils.dataToPublicKey(publicKey)
  }, options);
};

/**
 * @param {Buffer|string} data
 * @return {SetCandidateOnTxData}
 */
SetCandidateOnTxData.fromRlp = function fromRlp(data) {
  return SetCandidateOnTxData.fromBufferFields(new minterjsTx.TxDataSetCandidateOn(data));
};

module.exports = SetCandidateOnTxData;
