'use strict';

var minterjsTx = require('minterjs-tx');
var utils = require('../utils.js');

/**
 * @param {object} txTata
 * @param {number|string} txTata.price
 * @param {TxOptions} [options]
 * @constructor
 */
function PriceVoteTxData(_ref) {
  var price = _ref.price;
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  if (!options.disableValidation) {
    utils.validateUint(price, 'price');
  }
  this.price = price;
  this.txData = new minterjsTx.TxDataPriceVote({
    price: utils.integerToHexString(price)
  });
  utils.proxyNestedTxData(this);
}

/**
 * @param {Buffer|string|number} price
 * @param {TxOptions} [options]
 * @return {PriceVoteTxData}
 */
PriceVoteTxData.fromBufferFields = function fromBufferFields(_ref2) {
  var price = _ref2.price;
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return new PriceVoteTxData({
    price: utils.dataToInteger(price)
  }, options);
};

/**
 * @param {Buffer|string} data
 * @return {PriceVoteTxData}
 */
PriceVoteTxData.fromRlp = function fromRlp(data) {
  return PriceVoteTxData.fromBufferFields(new minterjsTx.TxDataPriceVote(data));
};

module.exports = PriceVoteTxData;
